<?php

/**
 * Full-Text Search class.
 *
 * @since 1.0.0
 */
class Full_Text_Search {
	const TABLE_NAME        = 'full_text_search_posts';
	const CUSTOM_FIELD_NAME = 'full_text_search_search_text';

	public $options;
	public $admin;
	public $pdfparser;

	/**
	 * Constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		load_plugin_textdomain( 'full-text-search' );

		register_activation_hook( __FILE__, array( $this, 'activation' ) );

		$this->options = get_option( 'full_text_search_options' );
		if ( false === $this->options || version_compare( $this->options['plugin_version'], FULL_TEXT_SEARCH_VERSION, '<' ) ) {
			$this->activation();
			$this->options = get_option( 'full_text_search_options' );
		}

		if ( false === $this->options ) {
			add_action( 'admin_notices', array( $this, 'admin_notices' ) );
			return;
		}

		require_once __DIR__ . '/admin.php';
		$this->admin = new Full_Text_Search_Admin( $this );

		add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
	}

	/**
	 * Fires once activated plugins have loaded.
	 *
	 * @since 1.0.0
	 */
	public function plugins_loaded() {
		add_action( 'full_text_search_event', array( $this, 'update_index_data' ) );
		add_action( 'wp_insert_post', array( $this, 'update_post' ), 10, 3 );
		add_action( 'add_attachment', array( $this, 'add_attachment' ), 10 );
		add_action( 'attachment_updated', array( $this, 'update_attachment' ), 10, 3 );
		add_action( 'deleted_post', array( $this, 'delete_post' ), 100, 1 );
		add_filter( 'posts_search', array( $this, 'posts_search' ), 10, 2);
		add_filter( 'posts_join', array( $this, 'posts_join' ), 10, 2 );
		add_filter( 'posts_orderby', array( $this, 'posts_orderby' ), 10, 2 );

		if ( isset( $this->options['enable_attachment'] ) && 'disable' !== $this->options['enable_attachment'] ) {
			add_action( 'pre_get_posts', array( $this, 'pre_get_posts' ) );
			add_filter( 'posts_where', array( $this, 'posts_where' ), 10, 2 );
		}

		if ( ! isset( $this->options['auto_pdf'] ) || $this->options['auto_pdf'] ) {
			require_once __DIR__ . '/lib/pdfparser.php';
			$this->pdfparser = null;
		}
	}

	/**
	 * Prints admin screen notices.
	 *
	 * @since 1.0.0
	 */
	public function admin_notices() {
		global $pagenow;

		if ( 'plugins.php' === $pagenow || 'index.php' === $pagenow ) {
			printf( '<div class="notice notice-warning"><p><b>%s</b>: %s</p></div>',
				esc_html( __( 'Full-Text Search', 'full-text-search' ) ),
				esc_html( __( 'Requires MySQL 5.6 or higher, or Mroonga engine.', 'full-text-search' ) )
			);
		}
	}

	/**
	 * Filters the search SQL that is used in the WHERE clause of WP_Query.
	 *
	 * @since 1.0.0
	 *
	 * @param string   $search Search SQL for WHERE clause.
	 * @param WP_Query $query  The WP_Query instance (passed by reference).
	 * @return string Search SQL for WHERE clause.
	 */
	public function posts_search( $search, $query ) {
		return '';
	}

	/**
	 * Add attachments to the search query.
	 *
	 * @since 1.6.0
	 *
	 * @param WP_Query $query Query instance.
	 * @return void.
	 */
	public function pre_get_posts( $query ) {
		if ( $query->is_search ) {
			$query->set( 'post_status', array( 'publish', 'private', 'inherit' ) );
		}
	}

	/**
	 * Filters the JOIN clause of the query.
	 *
	 * @since 2.2.2
	 *
	 * @param string   $join  The JOIN clause of the query.
	 * @param WP_Query $query The WP_Query instance (passed by reference).
	 * @return string JOIN clause.
	 */
	public function posts_join( $join, $query ) {
		global $wpdb;

		if ( $query->is_search ) {
			$table_name = $wpdb->prefix . Full_Text_Search::TABLE_NAME;

			$enable_mode = isset( $this->options['enable_mode'] ) ? $this->options['enable_mode'] : 'enable';

			if (
				( 'enable' === $enable_mode ) ||
				( 'search' === $enable_mode && $query->is_main_query() && is_search() )
			) {
				$table_name = $wpdb->prefix . Full_Text_Search::TABLE_NAME;
				$boolean_logic = isset( $this->options['boolean_logic'] ) ? $this->options['boolean_logic'] : 'and';

				$s = trim( sanitize_text_field( $query->query_vars['s'] ) );
				$s = str_replace( array( '*', ';' ), '', $s );

				if ( '' === $s ) {
					return $join;
				}

				if ( 'mroonga' === $this->options['db_engine'] ) {
					$d = ( 'and' === $boolean_logic ) ? 'D+' : 'DOR';
					$s = "*{$d}W1:10,2:1 " . $s;
				} else {
					if ( 'and' === $boolean_logic ) {
						$keywords = preg_split( "/[\s,]*(\\\"[^\"]+\")[\s,]*|" . "[\s,]*(\\'[^\\']+\\')[\s,]*|" . "[\s,]+/", $s, 0, PREG_SPLIT_NO_EMPTY | PREG_SPLIT_DELIM_CAPTURE );
						if ( count( $keywords ) > 1 ) {
							for ( $i = 0 ; $i < count( $keywords ); $i++ ) {
								if ( ! in_array( substr( $keywords[$i], 0, 1), array( '-', '+', '~' ) ) ) {
									$keywords[$i]= '+' . $keywords[$i];
								}
							}
							$s = implode( ' ', $keywords );
						}
					}
				}

				$join .= $wpdb->prepare(
					" INNER JOIN (" .
					"SELECT ID, MATCH(keywords, post_title, post_content, post_excerpt) AGAINST(%s IN BOOLEAN MODE) AS score " .
					"FROM {$table_name} WHERE MATCH(keywords, post_title, post_content, post_excerpt) AGAINST(%s IN BOOLEAN MODE)" .
					") matched_posts ON matched_posts.ID = {$wpdb->posts}.ID",
					$s, $s
				);
			}
		}

		return $join;
	}

	/**
	 * Filters the WHERE clause of the query.
	 *
	 * @since 1.6.0
	 *
	 * @param string   $where The WHERE clause of the query.
	 * @param WP_Query $query The WP_Query instance (passed by reference).
	 * @return string WHERE clause.
	 */
	public function posts_where( $where, $query ) {
		global $wpdb;

		if ( $query->is_search ) {
			$where .= " AND ({$wpdb->posts}.post_status = 'publish' OR {$wpdb->posts}.post_status = 'inherit'";
			if ( is_user_logged_in() ) {
				$post_type = $query->query_vars['post_type'];

				if ( is_array( $post_type ) && count( $post_type ) > 1 ) {
					$post_type_cap = 'multiple_post_type';
				} else {
					if ( is_array( $post_type ) ) {
						$post_type = reset( $post_type );
					}
					$post_type_object = get_post_type_object( $post_type );
					if ( empty( $post_type_object ) ) {
						$post_type_cap = $post_type;
					}
				}

				if ( 'any' === $post_type ) {
				} elseif ( ! empty( $post_type ) && is_array( $post_type ) ) {
				} elseif ( ! empty( $post_type ) ) {
					$post_type_object = get_post_type_object( $post_type );
				} elseif ( $query->is_attachment ) {
					$post_type_object = get_post_type_object( 'attachment' );
				} elseif ( $query->is_page ) {
					$post_type_object = get_post_type_object( 'page' );
				} else {
					$post_type_object = get_post_type_object( 'post' );
				}
		
				if ( ! empty( $post_type_object ) ) {
					$read_private_cap = $post_type_object->cap->read_private_posts;
				} else {
					$read_private_cap = 'read_private_' . $post_type_cap . 's';
				}

				$user_id = get_current_user_id();
				$private_states = get_post_stati( array( 'private' => true ) );
				foreach ( (array) $private_states as $state ) {
					$where .= current_user_can( $read_private_cap ) ? " OR {$wpdb->posts}.post_status = '$state'" : " OR {$wpdb->posts}.post_author = $user_id AND {$wpdb->posts}.post_status = '$state'";
				}
			}
			$where .= ')';

			if ( isset( $this->options['enable_attachment'] ) && 'filter' === $this->options['enable_attachment'] ) {
				$post_mime_types = array();
				if ( isset( $this->options['enable_zip'] ) && $this->options['enable_zip'] ) {
					$post_mime_types[] = "{$wpdb->posts}.post_mime_type = 'application/zip'";
				}
				if ( isset( $this->options['enable_pdf'] ) && $this->options['enable_pdf'] ) {
					$post_mime_types[] = "{$wpdb->posts}.post_mime_type = 'application/pdf'";
				}
				if ( isset( $this->options['enable_word'] ) && $this->options['enable_word'] ) {
					$post_mime_types[] = "{$wpdb->posts}.post_mime_type = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'";
					$post_mime_types[] = "{$wpdb->posts}.post_mime_type = 'application/msword'";
				}
				if ( isset( $this->options['enable_excel'] ) && $this->options['enable_excel'] ) {
					$post_mime_types[] = "{$wpdb->posts}.post_mime_type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'";
				}
				if ( isset( $this->options['enable_powerpoint'] ) && $this->options['enable_powerpoint'] ) {
					$post_mime_types[] = "{$wpdb->posts}.post_mime_type = 'application/vnd.openxmlformats-officedocument.presentationml.presentation'";
				}

				if ( count( $post_mime_types ) ) {
					$post_mime_type = implode( ' OR ', $post_mime_types );
					$where .= " AND ({$wpdb->posts}.post_type <> 'attachment' OR {$wpdb->posts}.post_type = 'attachment' AND ({$post_mime_type}))";
				}
			}
		}
		return $where;
	}

	/**
	 * Filters the ORDER BY clause of the query.
	 *
	 * @since 1.0.0
	 *
	 * @param string   $orderby The ORDER BY clause of the query.
	 * @param WP_Query $query   The WP_Query instance (passed by reference).
	 * @return string The ORDER BY clause of the query.
	 */
	public function posts_orderby( $orderby, $query ) {
		global $wpdb;

		if ( $query->is_search && ! empty( trim( $query->query_vars['s'] ) ) ) {
			if ( empty( $query->query_vars['orderby'] ) ) {
				$enable_mode = isset( $this->options['enable_mode'] ) ? $this->options['enable_mode'] : 'enable';
				if (
					( 'enable' === $enable_mode ) ||
					( 'search' === $enable_mode && $query->is_main_query() && is_search() )
				) {
					$orderby = 'score DESC';
				}
			}
		}
		return $orderby;
	}

	/**
	 * Update a post.
	 *
	 * @since 1.2.0
	 * @since 1.4.0 Removed $update parameter.
	 *
	 * @param int     $post_ID Post ID.
	 * @param WP_Post $post    Post object.
	 * @param bool    $update  Whether this is an existing post being updated.
	 * @return void
	 */
	public function update_post( $post_ID, $post, $updated ) {
		global $wpdb;

		$table_name = $wpdb->prefix . Full_Text_Search::TABLE_NAME;
		$post_types = get_post_types( array( 'exclude_from_search' => false ) );
		if ( in_array( $post->post_type, $post_types ) && in_array( $post->post_status, array( 'publish', 'private', 'inherit', 'trash' ) ) ) {
			$text = get_post_meta( $post_ID, Full_Text_Search::CUSTOM_FIELD_NAME, true );
			$status = 0;

			if ( 'attachment' == $post->post_type && empty( $text ) && ! $updated ) {
				if  ( 'application/pdf' == $post->post_mime_type ) {
					if ( isset( $this->options['auto_pdf'] ) ? $this->options['auto_pdf'] : true ) {
						$status = 1;
					}
				} else if ( 'application/vnd.openxmlformats-officedocument.wordprocessingml.document' == $post->post_mime_type || 'application/msword' == $post->post_mime_type ) {
					if ( isset( $this->options['auto_word'] ) ? $this->options['auto_word'] : true ) {
						$status = 1;
					}
				} else if ( 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' == $post->post_mime_type ) {
					if ( isset( $this->options['auto_excel'] ) ? $this->options['auto_excel'] : true ) {
						$status = 1;
					}
				} else if ( 'application/vnd.openxmlformats-officedocument.presentationml.presentation' == $post->post_mime_type ) {
					if ( isset( $this->options['auto_powerpoint'] ) ? $this->options['auto_powerpoint'] : true ) {
						$status = 1;
					}
				}
			}

			$wpdb->replace( $table_name, array(
				'ID'            => $post->ID,
				'post_type'     => $post->post_type,
				'post_status'   => $post->post_status,
				'post_password' => $post->post_password,
				'post_modified' => $post->post_modified,
				'post_title'    => wp_strip_all_tags( $post->post_title ),
				'post_content'  => wp_strip_all_tags( $post->post_content ),
				'post_excerpt'  => wp_strip_all_tags( $post->post_excerpt ),
				'keywords'      => $text,
				'status'        => $status
			) );

			if ( 1 === $status ) {
				$filename = get_attached_file( $post_ID );
				if ( file_exists( $filename ) ) {
					if ( 'application/pdf' === $post->post_mime_type ) {
						try {
							$text = $this->get_text_from_pdf_file( $filename );
							$status = 0;
						} catch ( \Exception $e ) {
							$status = 2;
						}
					} else if ( 'application/msword' === $post->post_mime_type ) {
						try {
							$text = $this->get_text_from_doc_file( $filename );
							$status = 0;
						} catch ( Exception $e ) {
							$error = $e->getMessage();
							if ( 'Unsupported file format.' === $error ) {
								$status = 3;
							} else {
								$status = 4;
							}
						}
					} else if ( 'application/vnd.openxmlformats-officedocument.wordprocessingml.document' === $post->post_mime_type ) {
						try {
							$text = $this->get_text_from_docx_file( $filename );
							$status = 0;
						} catch ( Exception $e ) {
							$status = 4;
						}
					} else if ( 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' === $post->post_mime_type ) {
						try {
							$text = $this->get_text_from_excel_file( $filename );
							$status = 0;
						} catch ( Exception $e ) {
							$status = 4;
						}
					} else if ( 'application/vnd.openxmlformats-officedocument.presentationml.presentation' === $post->post_mime_type ) {
						try {
							$text = $this->get_text_from_powerpoint_file( $filename );
							$status = 0;
						} catch ( Exception $e ) {
							$status = 4;
						}
					}

					if ( ! empty( $text ) ) {
						update_post_meta( $post_ID, Full_Text_Search::CUSTOM_FIELD_NAME, $text );
					}

					$wpdb->query( $wpdb->prepare( "UPDATE {$table_name} SET keywords=%s,status=%d WHERE ID=%d;",
						$text,
						$status,
						$post->ID
					) );
				}
			}
		}
	}

	/**
	 * Add attachment.
	 *
	 * @since 1.4.0
	 *
	 * @param int $post_ID Post ID.
	 * @return void
	 */
	public function add_attachment( $post_ID ) {
		$post = get_post( $post_ID );
		$this->update_post( $post_ID, $post, false );
	}

	/**
	 * Update attachment.
	 *
	 * @since 1.8.0
	 *
	 * @param int $post_ID Post ID.
	 * @return void
	 */
	public function update_attachment( $post_ID, $post_after, $post_before ) {
		$this->update_post( $post_ID, $post_after, true );
	}

	/**
	 * Delete a post.
	 *
	 * @since 1.2.0
	 *
	 * @param int $postid Post ID.
	 * @return void
	 */
	public function delete_post( $post_id ) {
		global $wpdb;

		$table_name = $wpdb->prefix . Full_Text_Search::TABLE_NAME;
		$wpdb->delete( $table_name, array( 'ID' => $post_id ) );
	}

	/**
	 * Gets the default value of the options.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function get_default_options() {
		global $wpdb;

		return array( 
			'plugin_version'    => FULL_TEXT_SEARCH_VERSION,
			'db_type'           => '',			// '', 'mariadb' or 'mysql',
			'db_engine'         => '',			// '', 'mroonga' or'innodb',
			'boolean_logic'     => 'and',		// 'and' or 'or'.
			'enable_mode'       => 'enable',	// 'disable', 'enable' or 'search'.
			'enable_attachment' => 'enable',	// 'disable', 'enable' or 'filter'.
			'enable_zip'        => false,
			'enable_pdf'        => false,
			'enable_word'       => false,
			'enable_excel'      => false,
			'enable_powerpoint' => false,
			'auto_pdf'          => true,
			'auto_word'         => true,
			'auto_excel'        => true,
			'auto_powerpoint'   => true,
		);
	}

	/**
	 * Create a search table.
	 *
	 * @since 1.0.0
	 *
	 * @param string $db_engine DB engine. Accepts 'mroonga' or 'innodb'.
	 * @return bool
	 */
	public function create_table( $db_engine ) {
		global $wpdb;

		if ( 'mroonga' !== $db_engine && 'innodb' !== $db_engine ) {
			return false;
		}

		$result = false;

		$db_index = ( 'mroonga' == $db_engine ) ?
			'FULLTEXT INDEX (keywords,post_title,post_content,post_excerpt) COMMENT \'parser "TokenMecab"\'' :
			'FULLTEXT INDEX (keywords,post_title,post_content,post_excerpt) WITH PARSER ngram';

		//$charset_collate = $wpdb->get_charset_collate();
		$charset_collate = 'DEFAULT CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci';

		$table_name = $wpdb->prefix . Full_Text_Search::TABLE_NAME;

		$table_engine = $wpdb->get_var( $wpdb->prepare( 'SELECT ENGINE FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME=%s', $table_name ) );
		if ( null === $table_engine ) {
			$result = $wpdb->query( "CREATE TABLE `{$table_name}` (
				ID bigint(20) unsigned NOT NULL auto_increment,
				post_type varchar(20) NOT NULL default 'post',
				post_status varchar(20) NOT NULL default 'publish',
				post_password varchar(255) NOT NULL default '',
				post_modified datetime NOT NULL default '0000-00-00 00:00:00',
				post_title text NOT NULL,
				post_content longtext NOT NULL,
				post_excerpt text NOT NULL,
				keywords longtext NOT NULL,
				status tinyint(1) NOT NULL default '0',
				PRIMARY KEY (ID),
				{$db_index}
				) ENGINE={$db_engine} {$charset_collate};"
			);
		} else if ( $db_engine != strtolower( $table_engine ) ) {
			$engine = ( 'mroonga' == $db_engine ) ? 'Mroonga' : 'InnoDB';
			$result = $wpdb->query( "ALTER TABLE {$table_name} ENGINE={$engine}" );
		} else {
			$result = true;
		}

		return $result;
	}

	/**
	 * Extracts text string from a PDF file.
	 *
	 * @since 2.0.0
	 *
	 * @throws Exception The file fails to load.
	 *
	 * @param string $file    Path to the file.
	 * @param string $newline Newline characters. Default is \n.
	 * @return string text string.
	 */
	public function get_text_from_pdf_file( $file, $newline = "\n" ) {
		$text = '';

		if ( ! isset( $this->pdfparser ) ) {
			$this->pdfparser = new \Smalot\PdfParser\Parser();
		}
		$pdffile = $this->pdfparser->parseFile( $file );
		$pages = $pdffile->getPages();
		foreach ( $pages as $page ) {
			$text .= $page->getText() . $newline;
		}
		$text = trim( $text );

		return $text;
	}

	/**
	 * Extracts text string from a Word (docx) file.
	 *
	 * @since 2.0.0
	 *
	 * @throws Exception The file fails to load.
	 *
	 * @param string $file    Path to the file.
	 * @param string $newline Newline characters. Default is \n.
	 * @return string text string.
	 */
	public function get_text_from_docx_file( $file, $newline = "\n" ) {
		$text = '';

		$zip = new ZipArchive();
		if ( $zip->open( $file ) === true ) {
			$xml = $zip->getFromName( 'word/document.xml' );
			if ( $xml ) {
				$dom = new DOMDocument();
				$dom->loadXML( $xml );
				$paragraphs = $dom->getElementsByTagName( 'p' );
				foreach ( $paragraphs as $p ) {
					$ts = $p->getElementsByTagName( 't' );
					foreach ( $ts as $t ) {
						$text .= $t->nodeValue;
					}
					$text .= $newline;
				}
				$text = trim( $text );
			}
			$zip->close();
		} else {
			throw new Exception( 'File open error.' );
		}

		return $text;
	}

	/**
	 * Extracts text string from a Word (doc) file.
	 *
	 * @since 2.0.0
	 *
	 * @throws Exception The file fails to load.
	 *
	 * @param string $file    Path to the file.
	 * @return string text string.
	 */
	public function get_text_from_doc_file( $file ) {
		$text = '';

		if ( ( $fh = fopen( $file, 'r' ) ) !== false ) {
			$headers = fread( $fh, 0xA00 );
			$len = ( ord( $headers[0x21C] ) - 1 );
			$len += ( ( ord( $headers[0x21D] ) - 8 ) * 256 );
			$len += ( ( ord( $headers[0x21E] ) * 256 ) * 256 );
			$len += ( ( ( ord( $headers[0x21F] ) * 256 ) * 256 ) * 256 );
			if ( $len > 0 ) {
				$text = fread( $fh, $len );
				$text = mb_convert_encoding( $text, 'UTF-8', 'UTF-16LE' );
				$text = preg_replace( '/[\x00-\x09\x0B\x0C\x0E-\x1F\x7F]/', '', $text );
				$text = trim( $text );
			}
			fclose( $fh );
			if ( $len <= 0 ) {
				throw new Exception( 'Unsupported file format.' );
			}
		} else {
			throw new Exception( 'File open error.' );
		}

		return $text;
	}

	/**
	 * Extracts text string from a Excel (xlsx) file.
	 *
	 * @since 2.1.0
	 *
	 * @throws Exception The file fails to load.
	 *
	 * @param string $file    Path to the file.
	 * @param string $newline Newline characters. Default is \n.
	 * @return string text string.
	 */
	public function get_text_from_excel_file( $file, $newline = "\n" ) {
		$text = '';

		$zip = new ZipArchive();
		if ( $zip->open( $file ) === true ) {
			$xml = $zip->getFromName( 'xl/sharedStrings.xml' );
			if ( $xml ) {
				$dom = new DOMDocument();
				$dom->loadXML( $xml );
				$ts = $dom->getElementsByTagName( 't' );
				foreach ( $ts as $t ) {
					$text .= $t->nodeValue;
					$text .= $newline;
				}
				$text = trim( $text );
			}
			$zip->close();
		} else {
			throw new Exception( 'File open error.' );
		}

		return $text;
	}

	/**
	 * Extracts text string from a PowerPoint (pptx) file.
	 *
	 * @since 2.1.0
	 *
	 * @throws Exception The file fails to load.
	 *
	 * @param string $file    Path to the file.
	 * @param string $newline Newline characters. Default is \n.
	 * @return string text string.
	 */
	public function get_text_from_powerpoint_file( $file, $newline = "\n" ) {
		$text = '';

		$zip = new ZipArchive();
		if ( $zip->open( $file ) === true ) {
			$dom = new DOMDocument();

			$i = 1;
			while ( false !== ( $xml = $zip->getFromName( "ppt/slides/slide{$i}.xml" ) ) ) {
				$dom->loadXML( $xml );
				$paragraphs = $dom->getElementsByTagName( 'p' );
				foreach ( $paragraphs as $p ) {
					$ts = $p->getElementsByTagName( 't' );
					foreach ( $ts as $t ) {
						$text .= $t->nodeValue;
					}
					$text .= $newline;
				}
				$i++;
			}

			$i = 1;
			while ( false !== ( $xml = $zip->getFromName( "ppt/diagrams/data{$i}.xml" ) ) ) {
				$dom->loadXML( $xml );
				$paragraphs = $dom->getElementsByTagName( 'p' );
				foreach ( $paragraphs as $p ) {
					$ts = $p->getElementsByTagName( 't' );
					foreach ( $ts as $t ) {
						$text .= $t->nodeValue;
					}
					$text .= $newline;
				}
				$i++;
			}

			$text = trim( $text );
			$zip->close();
		} else {
			throw new Exception( 'File open error.' );
		}

		return $text;
	}

	/**
	 * Update index data.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function update_index_data() {
		global $wpdb;

		$table_name = $wpdb->prefix . Full_Text_Search::TABLE_NAME;

		$post_types = get_post_types( array( 'exclude_from_search' => false ) );
		$sql_posts = "'" . implode( "','", array_map( 'esc_sql', $post_types ) ) . "'";

		$posts_count = (int) $wpdb->get_var( "SELECT COUNT(ID) FROM {$wpdb->posts} WHERE post_type IN ({$sql_posts}) AND post_status IN ('private','publish','inherit','trash');" );

		/**
		 * Maximum number of records to process at one time.
		 *
		 * @since 1.1.0
		 * @since 2.2.0 Added the `$posts_count` parameter.
		 *
		 * @param int $limit       Number of records. 0 to stop indexing.
		 * @param int $posts_count Number of search targets.
		 */
		$limit = (int) apply_filters( 'full_text_search_limit', 2000 >= $posts_count ? 100 : 1000, $posts_count );

		if ( 0 === $limit ) {
			return;
		}

		/*
		 * Delete
		 */
		$wpdb->query( "DELETE FROM {$table_name} WHERE ID NOT IN (SELECT ID FROM {$wpdb->posts} WHERE post_type IN ({$sql_posts}) AND post_status IN ('publish','private','inherit','trash'))" );

		/*
		 * Update
		 */
		$rows = $wpdb->get_results(
			"SELECT SQL_CALC_FOUND_ROWS t1.ID, t1.post_type, t1.post_status, t1.post_password, t1.post_modified, t1.post_title, t1.post_content, t1.post_excerpt " .
			"FROM {$wpdb->posts} AS t1 LEFT OUTER JOIN {$table_name} AS t2 ON (t1.ID = t2.ID) " . 
			"WHERE t1.post_modified > t2.post_modified LIMIT {$limit};"
		);

		$found_rows = (int) $wpdb->get_var( 'SELECT FOUND_ROWS()' );
		$found_rows = $found_rows - count( $rows );

		foreach ( $rows as $row ) {
			$wpdb->query( $wpdb->prepare( "UPDATE {$table_name} SET post_status=%s,post_password=%s,post_modified=%s,post_title=%s,post_content=%s,post_excerpt=%s WHERE ID=%d;",
				$row->post_status,
				$row->post_password,
				$row->post_modified,
				wp_strip_all_tags( $row->post_title ),
				wp_strip_all_tags( $row->post_content ),
				wp_strip_all_tags( $row->post_excerpt ),
				$row->ID
			) );
		}

		/*
		 * Insert
		 */
		if ( 0 >= $found_rows ) {
			$rows = $wpdb->get_results(
				"SELECT SQL_CALC_FOUND_ROWS t1.ID, t1.post_type, t1.post_mime_type, t1.post_status, t1.post_password, t1.post_modified, t1.post_title, t1.post_content, t1.post_excerpt, m.meta_value AS keywords " .
				"FROM {$wpdb->posts} AS t1 LEFT OUTER JOIN {$table_name} AS t2 ON (t1.ID = t2.ID) " .
				"LEFT JOIN {$wpdb->postmeta} AS m ON t1.ID = m.post_id AND m.meta_key = '" . Full_Text_Search::CUSTOM_FIELD_NAME . "' " . 
				"WHERE t2.ID IS NULL AND t1.post_type IN ({$sql_posts}) AND t1.post_status IN ('private','publish','inherit','trash') LIMIT {$limit};"
			);
			
			$found_rows = (int) $wpdb->get_var( 'SELECT FOUND_ROWS()' );
			$found_rows = $found_rows - count( $rows );

			$fields = 'ID,post_type,post_status,post_password,post_modified,post_title,post_content,post_excerpt,keywords,status';

			foreach ( $rows as $row ) {
				$text = $row->keywords;
				$status = 0;

				if ( 'attachment' == $row->post_type && empty( $text ) ) {
					if ( 'application/pdf' == $row->post_mime_type ) {
						if ( isset( $this->options['auto_pdf'] ) ? $this->options['auto_pdf'] : true ) {
							$status = 1;
						}
					} else if ( 'application/vnd.openxmlformats-officedocument.wordprocessingml.document' == $row->post_mime_type || 'application/msword' == $row->post_mime_type ) {
						if ( isset( $this->options['auto_word'] ) ? $this->options['auto_word'] : true ) {
							$status = 1;
						}
					} else if ( 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' == $row->post_mime_type ) {
						if ( isset( $this->options['auto_excel'] ) ? $this->options['auto_excel'] : true ) {
							$status = 1;
						}
					} else if ( 'application/vnd.openxmlformats-officedocument.presentationml.presentation' == $row->post_mime_type ) {
						if ( isset( $this->options['auto_powerpoint'] ) ? $this->options['auto_powerpoint'] : true ) {
							$status = 1;
						}
					}
				}

				$wpdb->query( $wpdb->prepare( "INSERT INTO {$table_name} ({$fields}) VALUES (%d,%s,%s,%s,%s,%s,%s,%s,%s,%d);",
					$row->ID,
					$row->post_type,
					$row->post_status,
					$row->post_password,
					$row->post_modified,
					wp_strip_all_tags( $row->post_title ),
					wp_strip_all_tags( $row->post_content ),
					wp_strip_all_tags( $row->post_excerpt ),
					$text,
					$status
				) );

				if ( 1 === $status ) {
					$filename = get_attached_file( $row->ID );
					if ( file_exists( $filename ) ) {
						if ( 'application/pdf' === $row->post_mime_type ) {
							try {
								$text = $this->get_text_from_pdf_file( $filename );
								$status = 0;
							} catch ( \Exception $e ) {
								$status = 2;
							}
						} else if ( 'application/msword' === $row->post_mime_type ) {
							try {
								$text = $this->get_text_from_doc_file( $filename );
								$status = 0;
							} catch ( Exception $e ) {
								$error = $e->getMessage();
								if ( 'Unsupported file format.' === $error ) {
									$status = 3;
								} else {
									$status = 4;
								}
							}
						} else if ( 'application/vnd.openxmlformats-officedocument.wordprocessingml.document' === $row->post_mime_type ) {
							try {
								$text = $this->get_text_from_docx_file( $filename );
								$status = 0;
							} catch ( Exception $e ) {
								$status = 4;
							}
						} else if ( 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' === $row->post_mime_type ) {
							try {
								$text = $this->get_text_from_excel_file( $filename );
								$status = 0;
							} catch ( Exception $e ) {
								$status = 4;
							}
						} else if ( 'application/vnd.openxmlformats-officedocument.presentationml.presentation' === $row->post_mime_type ) {
							try {
								$text = $this->get_text_from_powerpoint_file( $filename );
								$status = 0;
							} catch ( Exception $e ) {
								$status = 4;
							}
						}

						$wpdb->query( $wpdb->prepare( "UPDATE {$table_name} SET keywords=%s,status=%d WHERE ID=%d;",
							$text,
							$status,
							$row->ID
						) );
					}
				}
			}
		}

		if ( 0 < $found_rows ) {
			if ( ! wp_get_schedule( 'full_text_search_event' ) ) {
				wp_schedule_single_event( time() + 1, 'full_text_search_event' );
			}
		}
	}

	/**
	 * Get the search data by specifying the post ID.
	 *
	 * @since 1.8.0
	 *
	 * @global wpdb $wpdb WordPress database abstraction object.
	 *
	 * @param int $post_id Post ID
	 * @return object|null Database query result. null on failure.
	 */
	public function get_fulltext_row( $post_id ) {
		global $wpdb;

		$post_id = (int) $post_id;
		if ( ! $post_id ) {
			return null;
		}

		$_post = wp_cache_get( $post_id, 'fulltext_search_index_post' );

		if ( ! $_post ) {
			$table_name = $wpdb->prefix . Full_Text_Search::TABLE_NAME;
			$_post = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $table_name WHERE ID = %d LIMIT 1", $post_id ) );

			if ( ! $_post ) {
				return null;
			}

			wp_cache_add( $_post->ID, $_post, 'fulltext_search_index_post' );
		}

		return $_post;
	}

	/**
	 * Plugin activation.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function activation() {
		global $wp_version, $wpdb;

		if ( $wpdb->use_mysqli ) {
			$mysql_server_type = mysqli_get_server_info( $wpdb->dbh );
		} else {
			// @codingStandardsIgnoreStart
			$mysql_server_type = mysql_get_server_info( $wpdb->dbh );
			// @codingStandardsIgnoreEnd
		}

		$db_type = stristr( $mysql_server_type, 'mariadb' ) ? 'mariadb' : 'mysql';
		$db_version = $wpdb->get_var( 'SELECT VERSION()' );

		$enable_mroonga = $wpdb->get_var( "SELECT COUNT(*) FROM INFORMATION_SCHEMA.PLUGINS WHERE PLUGIN_NAME='Mroonga'" );
		if ( $enable_mroonga ) {
			$engine = 'mroonga';
		} else {
			if ( 'mysql' === $db_type && version_compare( '5.6', $db_version, '<=' ) ) {
				$engine = 'innodb';
			}
		}
		if ( ! isset( $engine ) ) {
			return;
		}

		$options = get_option( 'full_text_search_options' );
		if ( false === $options ) {
			$options = $this->get_default_options();
			$options['db_type'] = $db_type;
			$options['db_engine'] = $engine;
			add_option( 'full_text_search_options', $options );
		} else {
			// Less than 2.0.0
			if ( version_compare( $options['plugin_version'], '2.0.0', '<' ) ) {
				$table_name = $wpdb->prefix . Full_Text_Search::TABLE_NAME;
				$wpdb->query( "DROP TABLE IF EXISTS {$table_name};" );

				if ( isset( $options['enable_pdf_only'] ) ) {
					$options['enable_pdf'] = $options['enable_pdf_only'];
					unset( $options['enable_pdf_only'] );
				}
			}

			// Less than 2.1.0
			if ( version_compare( $options['plugin_version'], '2.1.0', '<' ) ) {
				if ( isset( $options['enable_attachment'] ) && $options['enable_attachment'] ) {
					$options['enable_attachment'] = ( $options['enable_pdf'] || $options['enable_word'] ) ? 'filter' : 'disable';
				} else {
					$options['enable_attachment'] = 'enable';
				}

				if ( isset( $options['enable_pdf_auto_text'] ) ) {
					$options['auto_pdf'] = $options['enable_pdf_auto_text'];
					unset( $options['enable_pdf_auto_text'] );
				}
				if ( isset( $options['enable_doc_auto_text'] ) ) {
					$options['auto_word'] = $options['enable_doc_auto_text'];
					unset( $options['enable_doc_auto_text'] );
				}
			}

			$options['plugin_version'] = FULL_TEXT_SEARCH_VERSION;
			$options['db_type'] = $db_type;
			$options['db_engine'] = $engine;
			update_option( 'full_text_search_options', $options );
		}

		$this->create_table( $engine );

		if ( ! wp_get_schedule( 'full_text_search_event' ) ) {
			wp_schedule_single_event( time() + 10, 'full_text_search_event' );
		}
	}

	/**
	 * Plugin Uninstall site.
	 *
	 * @since 1.8.0
	 *
	 * @return void
	 */
	public static function uninstall_site() {
		global $wpdb;

		if ( wp_get_schedule( 'full_text_search_event' ) ) {
			wp_clear_scheduled_hook( 'full_text_search_event' );
		}

		$table_name = $wpdb->prefix . Full_Text_Search::TABLE_NAME;
		if ( null !== $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $table_name ) ) ) ) {
			$wpdb->query( "DROP TABLE IF EXISTS {$table_name};" );
		}

		delete_metadata( 'post', 0, Full_Text_Search::CUSTOM_FIELD_NAME, false, true );

		delete_option( 'full_text_search_options' );
	}

	/**
	 * Plugin Uninstall all site.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public static function uninstall() {
		global $wpdb;

		if ( ! is_multisite() ) {
			Full_Text_Search::uninstall_site();
		} else {
			$current_blog_id = get_current_blog_id();
			$blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
			foreach ( $blog_ids as $blog_id ) {
				switch_to_blog( $blog_id );
				Full_Text_Search::uninstall_site();
			}
			switch_to_blog( $current_blog_id );
		}
	}
}
